@@ -1524,22 +1524,19 @@ class CommandV(Extensible):
         it pulls the latest changes, gets the different commits between two commits
         specified in the repos and ref_repos dictionary
         """
-        res = {'command': 'commit_history', 'result': {}}
         msg = self.__config.commit_history
+        res = {'command': 'commit_history', 'result': {}, 'build_id': msg.get('build_id')}
         name = msg.get('repo').get('name')
-        self.sender.send_message('Generating commit history of %s.' % (name))
         manager = git_helper.GitManager(
             self.__config.temp_working_folder,
             self.__config.deployer.get('commit_history_folder'), msg.get('ssh_key'))
+        logger.info('Generating commit history of %s.', name)
         history = manager.get_commit_history(msg.get('repo'), msg.get('ref'))
         res['result'] = history
-        message = 'Generated the commit history of %s.' % (name)
-        log_type = 'INFO'
         if history.get('error'):
-            message = 'Failed to generate the commit history of %s' % (name)
-            log_type = 'ERROR'
-            branch = msg.get('repo').get('branch')
             logger.error('Failed to generate the commit history of %s/%s: %s',
-                         name, branch, history.get('error'))
-        self.sender.send_message(message, body=res, log_type=log_type)
+                         name, msg.get('repo').get('branch'), history.get('error'))
+        else:
+            logger.info('Generated the commit history of %s.', name)
+        self.sender.send_message(body=res)
         self.done_trigger()
