@@ -59,7 +59,6 @@ Where:
 """
 
 import logging
-import time
 from os import path
 from docker import DockerClient
 from docker import APIClient as Client
@@ -69,12 +68,13 @@ from deployv.helpers import container
 from deployv.helpers.utils import get_error_message, decode
 from deployv.base import errors
 from retry_decorator import retry
+from time import sleep
 
 
 logger = logging.getLogger(__name__)  # pylint: disable=C0103
 
 
-class DockerV(object):
+class DockerV:
     """Helper class to make docker container deployments easier by code.
 
     If your docker service is not using the default socket you can change it by passing
@@ -191,8 +191,7 @@ class DockerV(object):
         try:
             inspected = self.inspect()
         except errors.NoSuchContainer as error:
-            logger.error("No such container: %s", self.docker_id)
-            logger.exception(get_error_message(error))
+            logger.debug(get_error_message(error))
             return None
 
         env_vars = inspected.get('Config').get('Env')
@@ -215,62 +214,22 @@ class DockerV(object):
             container_name)
         volume_binds = container.generate_binds(
             self.__config.get("volumes"), working_folder)
-        volumes = container.generate_volumes(self.__config.get("volumes"))
+        volumes = list(self.__config.get("volumes").values())
+        if self.config.get('full_stack') and self.config.get('pg_log_postgres'):
+            volume_binds.update({self.config.get('pg_log_postgres'): {
+                'bind': '/var/log/pg_log', 'ro': False}})
+            volumes.append('/var/log/pg_log')
         self.set_default_ports()
-        ports = container.generate_port_lists(self.__config.get("ports"))
-        port_bindings = container.generate_port_bindings(self.config.get('ports'))
-        container_config = {
-            "image": self.config.get("image_name"),
-            "command": self.config.get("command"),
-            "hostname": self.config.get("container_hostname"),
-            "ports": ports,
-            "environment": env_vars,
-            "volumes": volumes
-        }
-        if self.config.get("container_name", False):
-            container_config.update(
-                {'name': self.config.get("container_name").lower()})
-        host_config = {
-            "binds": volume_binds,
-            "port_bindings": port_bindings,
-            "mem_limit": self.config.get("mem_limit", 0),
-            "memswap_limit": self.config.get("mem_limit", 0),
-            "restart_policy": {
-                "MaximumRetryCount": 0,
-                "Name": "unless-stopped"
+        try:
+            self.create_container(env_vars, volume_binds, volumes)
+        except errors.ErrorPort:
+            if self.instance_type != 'develop':
+                raise
+            self.config['ports'] = {
+                port: 0 for port in self.config['ports']
             }
-        }
-        if self.__config.get("remove_previous", False):
-            self.__docker_id = self.config.get("container_name").lower()
-            self.remove_container()
-        hconfig = self.__cli.create_host_config(**host_config)
-        hconfig.get('PortBindings').get("5432/tcp", [{}])[0].\
-            update({'HostIp': '127.0.0.1'})
-        logger.debug('Container config %s', container_config)
-        try:
-            container_obj = self.__cli.create_container(
-                host_config=hconfig, **container_config)
-            self.__docker_id = container_obj.get("Id")
-            logger.debug('Created container %s', self.__docker_id)
-        except docker.errors.NotFound:
-            logger.exception('Image not found')
-            raise errors.NoSuchImage(self.config.get("image_name"))
-        except docker.errors.APIError as error:
-            logger.exception('Error creating the container')
-            if 'You have to remove (or rename) that container' in error.explanation:
-                logger.warning('The container %s already exists, skipping create step',
-                               container_config.get('name'))
-
-        try:
-            self.__cli.start(container=self.__docker_id)
-        except docker.errors.APIError as error:
-            if 'port is already allocated' in get_error_message(error) \
-                    or 'address already in use' in get_error_message(error):
-                logger.warning('Port already allocated')
-                raise errors.ErrorPort(
-                    'At least one of the ports used by this container are being used by another'
-                    ' process, manual intervention is required. Error message:\n{msg}'
-                    .format(msg=error.explanation))
+            self.set_default_ports()
+            self.create_container(env_vars, volume_binds, volumes)
         return self.basic_info
 
     def remove_container(self, container_name=False):
@@ -358,8 +317,9 @@ class DockerV(object):
         :returns: Installed packages
         :rtype: dict
         """
+        # pylint: disable=R1717
         pip_res = self.exec_cmd(['pip', 'freeze'])
-        pip = dict([line.split('==') for line in pip_res.split('\n') if '==' in line])
+        pip = dict([line.split('==') for line in pip_res.strip().split('\n') if '==' in line])
         apt_res = self.exec_cmd(['apt', 'list', '--installed'])
         apt = dict([(pck.split('/')[0] for pck in line.split(' ')[:2])
                     for line in apt_res.split('\n') if 'installed' in line])
@@ -401,12 +361,13 @@ class DockerV(object):
         :returns: Information about the container.  See `inspect_container in the official docs
             <https://docker-py.readthedocs.org/en/latest/api/#inspect_container>`_.
         """
+        container_id = container_id or self.docker_id
         try:
-            res = self.cli.inspect_container(container_id if container_id else self.docker_id)
+            res = self.cli.inspect_container(container_id)
         except docker.errors.NotFound:
-            raise errors.NoSuchContainer(container_id if container_id else self.docker_id)
+            raise errors.NoSuchContainer(container_id)
         except docker.errors.NullResource:
-            raise errors.NoSuchContainer(container_id if container_id else self.docker_id)
+            raise errors.NoSuchContainer(container_id)
         return res
 
     def install_packages(self, apt_packages=None, pip_packages=None):
@@ -516,17 +477,24 @@ class DockerV(object):
         return res
 
     @retry(docker.errors.APIError, tries=3, timeout_secs=0.5, logger=logger)
-    def pull(self, image_name):
+    def pull(self, image_name=False, raise_exception=False):
         """This is a simple wrapper for docker pull to use the deployv exceptions instead of the
         docker ones.
 
         :param image_name: The image name to pull.
         :type image_name: str
+        :param raise_execption: Identificator to know if raise the exepction.
+        :type raise_execption: bool
         """
+        image_name = image_name or self.config.get('image_name')
         try:
             self.cli.pull(image_name)
         except docker.errors.NotFound:
-            raise errors.NoSuchImage(image_name)
+            logger.debug('Image not found in hub')
+            if raise_exception:
+                raise errors.NoSuchImage(image_name)
+            image = self.inspect_image(image_name)
+            logger.debug('Image info %s', image)
 
     @retry(docker.errors.APIError, tries=3, timeout_secs=0.5, logger=logger)
     def push(self, image_name):
@@ -661,34 +629,6 @@ class DockerV(object):
             raise errors.NoSuchImage(image_name)
         return res
 
-    def restart_container(self):
-        # Get the information of the container
-        try:
-            container_info = self.basic_info
-        except errors.NoSuchContainer:
-            logger.error('Failed to restart the container, it does not exist')
-            return False
-        logger.info('Restarting container %s', container_info.get('name'))
-        # Restart container
-        try:
-            container_obj = self.cli2.containers.get(container_info.get('Id'))
-            container_obj.restart()
-        except docker.errors.APIError:
-            logger.error('Failed to restart the container, it took to long to start')
-            return False
-        # Get the container object again so it gets the updated state and
-        # check if it is running
-        container_obj = self.cli2.containers.get(container_info.get('Id'))
-        retries = 0
-        while retries <= 3:
-            if container_obj.status == 'running':
-                return True
-            time.sleep(5)
-            retries += 1
-        # If the while loop ends, that means the container failed to start
-        logger.error('Failed to restart the container, state: %s', container_obj.status)
-        return False
-
     def set_default_ports(self):
         """Sets default port mappings if the ones specified in the json
         contain falsy values so we can have control over the ports assigned
@@ -711,22 +651,96 @@ class DockerV(object):
             raise errors.ErrorPort('There are no available ports in the range'
                                    ' {start} - {end} to deploy this instance'
                                    .format(start=port_start, end=port_end))
-        instance_type = self._InstanceV__full_config.instance_config.get(
-            'instance_type')
         ports_to_map = [
             port for port, mapping in instance_ports.items() if not mapping
         ]
-        if instance_type == 'develop':
-            ports_to_map += [
-                port for port, mapping in
-                instance_ports.items() if mapping not in available_ports
-            ]
         if not ports_to_map:
             # There are no ports to map, try to use the ones specified
             return
         instance_ports.update(dict(zip(ports_to_map, available_ports)))
         return
 
+    def create_container(self, env_vars, volume_binds, volumes):
+        """Tries to create a container using the the given configuration as well
+         as the configuration contained in the calss instance
+
+        :param env_vars: List of env vars, vars names are uppercased as most of
+         unixlike OS ["VAR1=value1", "VAR2=234"]
+        :param volume_binds: Dict with the following format:
+
+        .. code-block:: python
+
+            {
+                "{{ working_folder }}/host_folder_name": {
+                    "bind": "/volume_inside_docker",
+                    "ro": False
+                    },
+                "{{ working_folder }}/host_folder_name_2": {
+                    "bind": "/volume_inside_docker_2",
+                    "ro": False
+                    }
+            }
+
+        :param volumes: List of volumes inside docker
+         ["volume_inside_docker", "volume_inside_docker_2"]
+         """
+        ports = container.generate_port_lists(self.__config.get("ports"))
+        port_bindings = container.generate_port_bindings(self.config.get('ports'))
+        container_config = {
+            "image": self.config.get("image_name"),
+            "command": self.config.get("command"),
+            "hostname": self.config.get("container_hostname"),
+            "ports": ports,
+            "environment": env_vars,
+            "volumes": volumes
+        }
+        if self.config.get("container_name", False):
+            container_config.update(
+                {'name': self.config.get("container_name").lower()})
+        host_config = {
+            "binds": volume_binds,
+            "port_bindings": port_bindings,
+            "mem_limit": self.config.get("mem_limit", 0),
+            "memswap_limit": self.config.get("mem_limit", 0),
+            "restart_policy": {
+                "MaximumRetryCount": 0,
+                "Name": "unless-stopped"
+            }
+        }
+        if self.instance_type == "develop":
+            host_config["cap_add"] = ["SYS_PTRACE"]
+        if self.__config.get("remove_previous", False):
+            self.__docker_id = self.config.get("container_name").lower()
+            self.remove_container()
+        hconfig = self.__cli.create_host_config(**host_config)
+        hconfig.get('PortBindings').get("5432/tcp", [{}])[0].\
+            update({'HostIp': '127.0.0.1'})
+        logger.debug('Container config %s', container_config)
+        try:
+            container_obj = self.__cli.create_container(
+                host_config=hconfig, **container_config)
+            self.__docker_id = container_obj.get("Id")
+            logger.debug('Created container %s', self.__docker_id)
+        except docker.errors.NotFound:
+            logger.exception('Image not found')
+            raise errors.NoSuchImage(self.config.get("image_name"))
+        except docker.errors.APIError as error:
+            logger.exception('Error creating the container')
+            if 'You have to remove (or rename) that container' in error.explanation:
+                logger.warning('The container %s already exists, skipping create step',
+                               container_config.get('name'))
+        try:
+            self.__cli.start(container=self.__docker_id)
+        except docker.errors.APIError as error:
+            error_message = get_error_message(error)
+            if 'port is already allocated' in error_message \
+                    or 'address already in use' in error_message:
+                logger.warning('Port already allocated')
+                raise errors.ErrorPort(
+                    'At least one of the ports used by this container are being used by another'
+                    ' process, manual intervention is required. Error message:\n{msg}'
+                    .format(msg=error.explanation))
+
     def _get_used_ports(self):
         """Method that checks the ports of all the containers in the server
         in order to list all the ports used by instances in order to decide
@@ -771,3 +785,45 @@ class DockerV(object):
             res.update(parsed_ports if
                        isinstance(parsed_ports, list) else [parsed_ports])
         return list(res)
+
+    def exec_cmd_image(self, command, volumes, working_folder=False, image=False):
+        """Execute the command in a specify image, also it binds the volumes with
+        the working folder to store the information that doesn't want to delete
+
+        :param command: The command that will execute in the image.
+        :type: str
+        :param working_folder: folder in the host where the volumes mounted
+            inside the container will be created.
+        :type: str
+        :param volumes: The volumes that will bind.
+        :type: dict with the format: {"host_folder_name": "volume_inside_docker"}
+        :param image: The image where will execute the command.
+        :type: str
+        """
+        image = image or self.config.get('image_name')
+        self.pull(image)
+        volumes = volumes or self.config.get('volumes')
+        volume_binds = volumes if not working_folder else container.generate_binds(
+            volumes, working_folder)
+        logger.debug('Executing command: %s', command)
+        cont = self.cli2.containers.run(image, command, volumes=volume_binds, detach=True)
+        while cont.status != 'exited':
+            logger.debug(
+                'Waiting for the command to finish, current state: %s', cont.status)
+            sleep(5)
+            cont.reload()
+        cont.remove()
+
+    def get_env_from_image(self, image=False):
+        """Get the environment of a specific image or use the image set in the config,
+        then parse the enviroment data in a dictionary.
+
+        :param image: The image's name where get the enviroment.
+        :type: str
+        :return: return the enviroment parse in a dictionary.
+        :rtype: dict
+        """
+        image = image or self.config.get('image_name')
+        info = self.inspect_image(image)
+        envs = info.get('Config').get('Env')
+        return container.parse_env_vars(envs)
